var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import semver from 'semver';
import { execAndLogStdoutOrThrowError } from './exec-and-log-stdout-or-throw-error';
/**
 * Installs a package
 *
 * @throws {@link Error} if the version is invalid
 * @throws {@link Error} if the exit code of npm install is not 0
 *
 * @example
```typescript
await npmInstall({
   packageName: 'typescript',
   version: 'latest',
   deepJsonFilePath,
   packageJsonFilePath,
})
```
 */
export function npmInstall(param) {
    return __awaiter(this, void 0, void 0, function* () {
        const { name: name, version, deepJsonFilePath, packageJsonFilePath } = param;
        if (version && !semver.valid(version))
            throw new Error(`Invalid version ${version}`);
        let npmInstallCommand = `npm install ${name}`;
        if (version) {
            npmInstallCommand += `@${version}`;
        }
        yield execAndLogStdoutOrThrowError({
            command: npmInstallCommand,
        });
        const { default: packageJson } = yield import(packageJsonFilePath, { assert: { type: 'json' } });
        const packageJsonDependencyVersion = packageJson.dependencies[name];
        const { default: deepJson } = yield import(deepJsonFilePath, { assert: { type: 'json' } });
        const deepJsonDependencyIndex = deepJson.dependencies.findIndex((dependency) => dependency.name === name);
        deepJson.dependencies[deepJsonDependencyIndex].version =
            packageJsonDependencyVersion;
    });
}
