var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import exec from '@simplyhexagonal/exec';
import { move } from 'fs-extra';
import path from 'path';
import { glob } from 'glob';
/**
 * Pulls the latest version of the npm package and copies it to the root folder
 *
 * @throws {@link Error} if there are not commited changes
 * @throws {@link Error} if the exit code of npm install is not 0
 *
 * @example
```typescript
await npmPull({
  packageName: '@deep-foundation/deep-memo',
})
```
 *
 */
export function npmPull(param) {
    return __awaiter(this, void 0, void 0, function* () {
        const { packageName } = param;
        const { execPromise: gitDiffExecPromise } = exec(`git diff`);
        const gitDiffResult = yield gitDiffExecPromise;
        if (gitDiffResult.stdoutOutput) {
            throw new Error('You have unstaged changes. Stash (git stash) or commit (git commit) them');
        }
        const { execPromise: npmInstallExecPromise } = exec(`npm install ${packageName}@latest --no-save`);
        const npmInstallResult = yield npmInstallExecPromise;
        if (npmInstallResult.exitCode !== 0) {
            throw new Error(npmInstallResult.stderrOutput.trim());
        }
        const rootFolderPath = path.resolve(`./`);
        const nodeModuleDirectoryPath = path.join(path.resolve(`node_modules`), packageName);
        const nodeModulePath = path.resolve(`node_modules/${packageName}`);
        const nodeModuleFilePaths = yield glob(`${nodeModulePath}/**/*`, {
            ignore: [`dist`, `node_modules`],
            withFileTypes: true,
        });
        yield Promise.all(nodeModuleFilePaths.map((nodeModuleFilePath) => __awaiter(this, void 0, void 0, function* () {
            if (!nodeModuleFilePath.isFile())
                return;
            return yield move(nodeModuleFilePath.fullpath(), path.join(rootFolderPath, nodeModuleFilePath.fullpath().replace(nodeModuleDirectoryPath, '')), {
                overwrite: true,
            });
        })));
    });
}
