var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import exec from '@simplyhexagonal/exec';
import { execAndLogStdoutOrThrowError } from './exec-and-log-stdout-or-throw-error';
import { updateDeepJsonVersion } from './update-version-in-json-object';
import { syncDependencies } from './sync-dependencies';
/**
 * Releases a new version of the deep npm package and syncronizes the version and dependencies between {@link NpmReleaseParam.deepJsonFilePath} and {@link NpmReleaseParam.packageJsonFilePath}
 *
 * @throws {@link Error} if the version in {@link NpmReleaseParam.packageJsonFilePath} is outdated
 *
 * @example
```typescript
await npmRelease({
  newVersion: '1.0.0',
  packageJsonFilePath: 'package.json',
  deepJsonFilePath: 'deep.json',
})
```
 *
 */
export function npmRelease(param) {
    return __awaiter(this, void 0, void 0, function* () {
        const { deepJsonFilePath, newVersion, packageJsonFilePath: packageJsonPath, } = param;
        yield syncDependencies({ deepJsonFilePath, packageJsonFilePath: packageJsonPath });
        const packageJson = yield import(packageJsonPath);
        const { execPromise } = exec(`npm view ${packageJson.name} version`);
        const execResult = yield execPromise;
        if (execResult.exitCode !== 0) {
            throw new Error(execResult.stderrOutput.trim());
        }
        const npmLatestPackageJsonVersion = execResult.stdoutOutput.toString().trim();
        const packageJsonVersion = packageJson.version;
        if (npmLatestPackageJsonVersion > packageJsonVersion) {
            throw new Error(`Version ${packageJson.version} in ${packageJsonPath} is outdated. Latest version in npm is ${npmLatestPackageJsonVersion}. Execute npm-pull`);
        }
        if (npmLatestPackageJsonVersion <= packageJsonVersion) {
            const { execResult: { stdoutOutput: npmVersionStdoutOutput }, } = yield execAndLogStdoutOrThrowError({
                command: `npm version --allow-same-version --no-git-tag-version ${newVersion}`,
            });
            yield updateDeepJsonVersion({
                version: npmVersionStdoutOutput.trimEnd().slice(1),
                filePath: deepJsonFilePath,
            });
        }
    });
}
