var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import fsExtra from 'fs-extra';
import semver from 'semver';
/**
 * Syncronizes dependencies between {@link SyncDependenciesParam.deepJsonFilePath} and {@link SyncDependenciesParam.packageJsonFilePath}
 *
 */
export function syncDependencies(param) {
    return __awaiter(this, void 0, void 0, function* () {
        const { deepJsonFilePath, packageJsonFilePath: packageJsonPath, } = param;
        const { default: deepJson } = yield import(deepJsonFilePath, { assert: { type: 'json' } });
        const { default: packageJson } = yield import(packageJsonPath, { assert: { type: 'json' } });
        if (!packageJson.dependencies) {
            packageJson.dependencies = {};
        }
        const missingDependenciesFromDeepJson = deepJson.dependencies.filter((dependency) => !!packageJson.dependencies[dependency.name]);
        missingDependenciesFromDeepJson.forEach((dependency) => {
            packageJson.dependencies = Object.assign(Object.assign({}, packageJson.dependencies), { [dependency.name]: `~${dependency.version}` });
        });
        if (missingDependenciesFromDeepJson.length > 0) {
            yield fsExtra.writeFile(packageJsonPath, JSON.stringify(packageJson, null, 2));
            console.log(`${missingDependenciesFromDeepJson.map((dependency) => dependency.name).join(', ')} are added to package.json because they exist in deep.json`);
        }
        deepJson.dependencies.forEach((dependency) => {
            const isDeepJsonVersionGreater = semver.gt(dependency.version, packageJson.dependencies[dependency.name]);
            if (isDeepJsonVersionGreater) {
                packageJson.dependencies[dependency.name] = `~${dependency.version}`;
            }
            else {
                deepJson.dependencies = Object.assign(Object.assign({}, deepJson.dependencies), { [dependency.name]: `${dependency.version}` });
            }
        });
        Object.entries(packageJson.dependencies).forEach(([dependencyName, dependencyVersion]) => {
            const deepJsonDependency = deepJson.dependencies.find(dependency => dependency.name === dependency.name);
            if (!deepJsonDependency)
                return;
            const isPackageJsonVersionGreater = semver.gt(dependencyVersion, deepJson.dependencies.find(dependency => dependency.name === dependency.name).version);
            if (isPackageJsonVersionGreater) {
                deepJson.dependencies = Object.assign(Object.assign({}, deepJson.dependencies), { [dependencyName]: `${dependencyVersion}` });
            }
            else {
                packageJson.dependencies[dependencyName] = `~${deepJsonDependency.version}`;
            }
        });
    });
}
